package ru.dbotthepony.kstarbound

import com.google.gson.Gson
import com.google.gson.JsonObject
import it.unimi.dsi.fastutil.objects.Object2ObjectFunction
import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap
import ru.dbotthepony.kstarbound.api.IStarboundFile
import ru.dbotthepony.kstarbound.defs.player.RecipeDefinition
import java.util.Collections

class RecipeRegistry {
	private val recipesInternal = ArrayList<RegistryObject<RecipeDefinition>>()
	private val group2recipesInternal = Object2ObjectOpenHashMap<String, ArrayList<RegistryObject<RecipeDefinition>>>()
	private val group2recipesBacking = Object2ObjectOpenHashMap<String, List<RegistryObject<RecipeDefinition>>>()
	private val output2recipesInternal = Object2ObjectOpenHashMap<String, ArrayList<RegistryObject<RecipeDefinition>>>()
	private val output2recipesBacking = Object2ObjectOpenHashMap<String, List<RegistryObject<RecipeDefinition>>>()
	private val input2recipesInternal = Object2ObjectOpenHashMap<String, ArrayList<RegistryObject<RecipeDefinition>>>()
	private val input2recipesBacking = Object2ObjectOpenHashMap<String, List<RegistryObject<RecipeDefinition>>>()

	val recipes: List<RegistryObject<RecipeDefinition>> = Collections.unmodifiableList(recipesInternal)
	val group2recipes: Map<String, List<RegistryObject<RecipeDefinition>>> = Collections.unmodifiableMap(group2recipesBacking)
	val output2recipes: Map<String, List<RegistryObject<RecipeDefinition>>> = Collections.unmodifiableMap(output2recipesBacking)
	val input2recipes: Map<String, List<RegistryObject<RecipeDefinition>>> = Collections.unmodifiableMap(input2recipesBacking)

	fun add(recipe: RegistryObject<RecipeDefinition>) {
		val value = recipe.value
		recipesInternal.add(recipe)

		for (group in value.groups) {
			group2recipesInternal.computeIfAbsent(group, Object2ObjectFunction { p ->
				ArrayList<RegistryObject<RecipeDefinition>>().also {
					group2recipesBacking[p as String] = Collections.unmodifiableList(it)
				}
			}).add(recipe)
		}

		output2recipesInternal.computeIfAbsent(value.output.item.name, Object2ObjectFunction { p ->
			ArrayList<RegistryObject<RecipeDefinition>>().also {
				output2recipesBacking[p as String] = Collections.unmodifiableList(it)
			}
		}).add(recipe)

		for (input in value.input) {
			input2recipesInternal.computeIfAbsent(input.item.name, Object2ObjectFunction { p ->
				ArrayList<RegistryObject<RecipeDefinition>>().also {
					input2recipesBacking[p as String] = Collections.unmodifiableList(it)
				}
			}).add(recipe)
		}
	}
}
